/*
 * Copyright 1995-2016 The OpenSSL Project Authors. All Rights Reserved.
 *
 * Licensed under the OpenSSL license (the "License").  You may not use
 * this file except in compliance with the License.  You can obtain a copy
 * in the file LICENSE in the source distribution or at
 * https://www.openssl.org/source/license.html
 */

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include "internal/numbers.h"
#include "internal/cryptlib.h"
#include <openssl/bio.h>

/*
 * Copyright Patrick Powell 1995
 * This code is based on code written by Patrick Powell <papowell@astart.com>
 * It may be used for any purpose as long as this notice remains intact
 * on all source code distributions.
 */


int BIO_printf(BIO *bio, const char *format, ...)
{
    va_list args;
    int ret;

    va_start(args, format);

    ret = BIO_vprintf(bio, format, args);

    va_end(args);
    return (ret);
}

static ssize_t cookie_BIO_write(void *cookie, const char *buf, size_t size)
{
        int ret;
        ret = BIO_write(cookie, buf, size);
        /* BIO_write may return negative value on error,
         * but we must return 0 on that case
         */
        return (ret < 0) ? 0 : ret;
}

int BIO_vprintf(BIO *bio, const char *format, va_list args)
{
    FILE *fp;
    int ret;

    cookie_io_functions_t  bio_funcs = {
        .read  = NULL,
        .write = cookie_BIO_write,
        .seek  = NULL,
        .close = NULL,
    };

    fp = fopencookie(bio, "w", bio_funcs);

    if (fp == NULL)
            return -1;

    ret = vfprintf(fp, format, args);

    fclose(fp);
    return (ret);
}

/*
 * As snprintf is not available everywhere, we provide our own
 * implementation. This function has nothing to do with BIOs, but it's
 * closely related to BIO_printf, and we need *some* name prefix ... (XXX the
 * function should be renamed, but to what?)
 */
int BIO_snprintf(char *buf, size_t n, const char *format, ...)
{
    va_list args;
    int ret;

    va_start(args, format);
    ret = vsnprintf(buf, n, format, args);
    va_end(args);

    if (ret >= n || ret == -1) return (-1);

    return (ret);
}

int BIO_vsnprintf(char *buf, size_t n, const char *format, va_list args)
{
    int ret;
    ret = vsnprintf(buf, n, format, args);

    if (ret >= n || ret == -1)
        return (-1);

    return (ret);
}
